import 'dart:math';

import 'package:flutter/material.dart';
//import 'package:flutter_screenutil/flutter_screenutil.dart';

typedef GradientProgressCallback = void Function(double progress);

class GradientProgress extends StatefulWidget {
  final GradientProgressDirection direction;
  final double progress;
  final double? strokeWidth;
  final List<Color>? colors;
  final List<Color>? bgColors;
  final double? radius;
  final Widget? thumb;
  final Size? thumbSzie;
  final GradientProgressCallback? onStartCallback;
  final GradientProgressCallback? onUpdateCallback;
  final GradientProgressCallback? onEndCallback;

  const GradientProgress({
    Key? key,
    required this.progress,
    this.direction = GradientProgressDirection.horizontal,
    this.strokeWidth,
    this.colors,
    this.bgColors,
    this.radius,
    this.thumb,
    this.thumbSzie,
    this.onStartCallback,
    this.onUpdateCallback,
    this.onEndCallback,
  }) : super(key: key);

  @override
  _GradientProgressState createState() => _GradientProgressState();
}

class _GradientProgressState extends State<GradientProgress> {
  final List<Color> _defultColors = [
    const Color(0xFF2E65E7),
    const Color(0xFF5EC2E0),
  ];
  final List<Color> _defultBgColors = [
    Colors.black,
  ];

  double _progress = 0;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, constraints) {
      double width = constraints.biggest.width;
      double height = constraints.biggest.height;
      double progress = min(1, max(0, widget.progress));
      CustomPainter painter;
      switch (widget.direction) {
        case GradientProgressDirection.circle:
          painter = _CircleProgressPaint(
            progress: progress,
            strokeWidth: widget.strokeWidth ?? 12,
            width: min(height, width),
            colors: widget.colors ?? _defultColors,
          );
          return GestureDetector(
            child: CustomPaint(painter: painter),
          );
        case GradientProgressDirection.horizontal:
          painter = _HorizontalProgressPainter(
            progress: progress,
            strokeWidth: widget.strokeWidth ?? 12,
            width: width,
            radius: widget.radius ?? 12,
            bgColors: widget.bgColors ?? _defultBgColors,
            colors: widget.colors ?? _defultColors,
          );
          return GestureDetector(
            child: CustomPaint(painter: painter),
            onHorizontalDragStart: (details) {
              if (details.localPosition.dx >= 0 &&
                  details.localPosition.dx <= width) {
                _progress = details.localPosition.dx / width;
                if (widget.onStartCallback != null) {
                  widget.onStartCallback!.call(_progress);
                }
              } else if (details.localPosition.dx < 0) {
                _progress = 0;
                if (widget.onStartCallback != null) {
                  widget.onStartCallback!.call(0);
                }
              } else if (details.localPosition.dx > width) {
                _progress = 1;
                if (widget.onStartCallback != null) {
                  widget.onStartCallback!.call(1);
                }
              }
            },
            onHorizontalDragUpdate: (details) {
              if (details.localPosition.dx >= 0 &&
                  details.localPosition.dx <= width) {
                _progress = details.localPosition.dx / width;
                if (widget.onUpdateCallback != null) {
                  widget.onUpdateCallback!.call(_progress);
                }
              } else if (details.localPosition.dx < 0) {
                _progress = 0;
                if (widget.onUpdateCallback != null) {
                  widget.onUpdateCallback!.call(0);
                }
              } else if (details.localPosition.dx > width) {
                _progress = 1;
                if (widget.onUpdateCallback != null) {
                  widget.onUpdateCallback!.call(1);
                }
              }
            },
            onHorizontalDragCancel: () {
              if (widget.onEndCallback != null) {
                widget.onEndCallback!.call(_progress);
              }
            },
            onHorizontalDragDown: (details) {
              if (details.localPosition.dx >= 0 &&
                  details.localPosition.dx <= width) {
                _progress = details.localPosition.dx / width;
                if (widget.onUpdateCallback != null) {
                  widget.onUpdateCallback!.call(_progress);
                }
              } else if (details.localPosition.dx < 0) {
                _progress = 0;
                if (widget.onUpdateCallback != null) {
                  widget.onUpdateCallback!.call(0);
                }
              } else if (details.localPosition.dx > width) {
                _progress = 1;
                if (widget.onUpdateCallback != null) {
                  widget.onUpdateCallback!.call(1);
                }
              }
            },
            onHorizontalDragEnd: (details) {
              if (widget.onEndCallback != null) {
                widget.onEndCallback!.call(_progress);
              }
            },
          );
        case GradientProgressDirection.vertical:
          painter = _VerticalProgressPainter(
            progress: progress,
            strokeWidth: widget.strokeWidth ?? 12,
            width: width,
            radius: widget.radius ?? 12,
            bgColors: widget.bgColors ?? _defultBgColors,
            colors: widget.colors ?? _defultColors,
          );
          return GestureDetector(
            child: Stack(
              children: [
                CustomPaint(
                  painter: painter,
                  size: Size(width, height),
                ),
                if (widget.thumb != null)
                  Container(
                    alignment: Alignment.topCenter,
                    margin: EdgeInsets.only(
                        top: (1 - progress) *
                            (height - (widget.thumbSzie?.height ?? 12))),
                    child: SizedBox(
                      width: widget.thumbSzie?.width ?? 12,
                      height: widget.thumbSzie?.height ?? 12,
                      child: widget.thumb!,
                    ),
                  ),
              ],
            ),
            onVerticalDragStart: (details) {
              if (details.localPosition.dy >= 0 &&
                  details.localPosition.dy <= height) {
                _progress = (height - details.localPosition.dy) / height;
                if (widget.onStartCallback != null) {
                  widget.onStartCallback!.call(_progress);
                }
              } else if (details.localPosition.dy < 0) {
                _progress = 0;
                if (widget.onStartCallback != null) {
                  widget.onStartCallback!.call(_progress);
                }
              } else if (details.localPosition.dy > height) {
                _progress = 1;
                if (widget.onStartCallback != null) {
                  widget.onStartCallback!.call(_progress);
                }
              }
            },
            onVerticalDragUpdate: (details) {
              if (details.localPosition.dy >= 0 &&
                  details.localPosition.dy <= height) {
                _progress = (height - details.localPosition.dy) / height;
                if (widget.onUpdateCallback != null) {
                  widget.onUpdateCallback!.call(_progress);
                }
              } else if (details.localPosition.dy < 0) {
                _progress = 1;
                if (widget.onUpdateCallback != null) {
                  widget.onUpdateCallback!.call(_progress);
                }
              } else if (details.localPosition.dy > height) {
                _progress = 0;
                if (widget.onUpdateCallback != null) {
                  widget.onUpdateCallback!.call(_progress);
                }
              }
            },
            onVerticalDragCancel: () {
              if (widget.onEndCallback != null) {
                widget.onEndCallback!.call(_progress);
              }
            },
            onVerticalDragDown: (details) {
              if (details.localPosition.dy >= 0 &&
                  details.localPosition.dy <= height) {
                _progress = (height - details.localPosition.dy) / height;
                if (widget.onUpdateCallback != null) {
                  widget.onUpdateCallback!.call(_progress);
                }
              } else if (details.localPosition.dy < 0) {
                _progress = 1;
                if (widget.onUpdateCallback != null) {
                  widget.onUpdateCallback!.call(_progress);
                }
              } else if (details.localPosition.dy > height) {
                _progress = 0;
                if (widget.onUpdateCallback != null) {
                  widget.onUpdateCallback!.call(_progress);
                }
              }
            },
            onVerticalDragEnd: (details) {
              if (widget.onEndCallback != null) {
                widget.onEndCallback!.call(_progress);
              }
            },
          );
      }
    });
  }
}

enum GradientProgressDirection { vertical, horizontal, circle }

class _CircleProgressPaint extends CustomPainter {
  final List<Color> colors;
  double progress;
  final double width;
  final double strokeWidth;
  final Paint circlePaint = Paint()
    ..style = PaintingStyle.stroke
    ..strokeWidth = 12;

  _CircleProgressPaint({
    required this.colors,
    required this.progress,
    required this.strokeWidth,
    required this.width,
  });

  @override
  void paint(Canvas canvas, Size size) {
    final radius =
        min(size.width, size.height) / 2 - circlePaint.strokeWidth / 2;
    final rect = Rect.fromCircle(
        radius: radius, center: Offset(size.width / 2, size.height / 2));
    circlePaint.shader = null;
    circlePaint.color = Colors.white;
    circlePaint.shader = SweepGradient(
      //startAngle: 0,
      //endAngle: pi * 2 * (progress == 0 ? 1 : progress),
      colors: colors,
      transform: const GradientRotation(-pi / 2),
    ).createShader(rect);
    circlePaint.strokeCap = StrokeCap.round;
    circlePaint.strokeWidth = strokeWidth;
    canvas.drawArc(rect, -pi / 2 + circlePaint.strokeWidth / width,
        pi * 2 * progress, false, circlePaint);
  }

  @override
  bool shouldRepaint(covariant _CircleProgressPaint oldDelegate) {
    return true;
  }
}

class _HorizontalProgressPainter extends CustomPainter {
  final List<Color> colors;
  final List<Color> bgColors;
  double progress;
  final double width;
  final double strokeWidth;
  final double radius;
  final Paint mPaint = Paint()..style = PaintingStyle.fill;
  final Paint mBgPaint = Paint()..style = PaintingStyle.fill;

  _HorizontalProgressPainter({
    required this.colors,
    required this.bgColors,
    required this.progress,
    required this.strokeWidth,
    required this.width,
    required this.radius,
  });

  @override
  void paint(Canvas canvas, Size size) {
    final bgRect = Rect.fromLTRB(0, 0, size.width, size.height);
    final bgRrect = RRect.fromRectXY(bgRect, radius, radius);
    if (bgColors.length == 1) {
      mBgPaint.shader = null;
      mBgPaint.color = bgColors[0];
    } else {
      mBgPaint.color = Colors.white;
      mBgPaint.shader = LinearGradient(
        colors: bgColors,
      ).createShader(bgRect);
    }
    if (colors.length == 1) {
      mPaint.shader = null;
      mPaint.color = colors[0];
    } else {
      mPaint.color = Colors.white;
      mPaint.shader = LinearGradient(
        colors: colors,
      ).createShader(bgRect);
    }

    mPaint.strokeWidth = strokeWidth;
    canvas.drawRRect(bgRrect, mBgPaint);

    double x = progress * size.width;
    if (x < radius) {
      double y = sqrt(radius * radius - (radius - x) * (radius - x));

      if (radius * 2 < size.height) {
        final rect1 = Rect.fromLTRB(0, radius - 1, x, size.height - radius + 1);
        canvas.drawRect(rect1, mPaint);
      }

      Path pathL = Path();

      final rectL = Rect.fromLTRB(0, 0, radius * 2, radius * 2);

      pathL.moveTo(0, radius);
      pathL.arcTo(rectL, pi, sin((x) / radius) * 2, false);
      pathL.lineTo(x, radius - y);
      pathL.lineTo(x, radius);
      canvas.drawPath(pathL, mPaint);

      Path pathR = Path();

      final rectR =
          Rect.fromLTRB(0, size.height - radius * 2, radius * 2, size.height);

      pathR.moveTo(0, size.height - radius);
      pathR.arcTo(rectR, pi, -sin((x) / radius) * 2, false);
      pathR.lineTo(x, size.height - radius + y);
      pathR.lineTo(x, size.height - radius);
      canvas.drawPath(pathR, mPaint);
    } else {
      final rect = Rect.fromLTRB(radius, 0, progress * size.width, size.height);
      final rrect = RRect.fromRectAndCorners(rect,
          topRight: Radius.circular(radius),
          bottomRight: Radius.circular(radius));
      canvas.drawRRect(rrect, mPaint);
      if (radius * 2 < size.width) {
        final rect1 =
            Rect.fromLTRB(0, radius - 1, radius, size.height - radius + 1);
        canvas.drawRect(rect1, mPaint);
      }

      final rectL = Rect.fromLTRB(0, 0, radius * 2, radius * 2);
      final rectR =
          Rect.fromLTRB(0, size.height - radius * 2, radius * 2, size.height);
      canvas.drawArc(rectL, pi, 0.5 * pi, true, mPaint);
      canvas.drawArc(rectR, pi, -0.5 * pi, true, mPaint);
    }
  }

  @override
  bool shouldRepaint(covariant _HorizontalProgressPainter oldDelegate) {
    return true;
  }
}

class _VerticalProgressPainter extends CustomPainter {
  final List<Color> colors;
  final List<Color> bgColors;
  double progress;
  final double width;
  final double strokeWidth;
  final double radius;
  final Paint mPaint = Paint()..style = PaintingStyle.fill;
  final Paint mBgPaint = Paint()..style = PaintingStyle.fill;

  _VerticalProgressPainter({
    required this.colors,
    required this.bgColors,
    required this.progress,
    required this.strokeWidth,
    required this.width,
    required this.radius,
  });

  @override
  void paint(Canvas canvas, Size size) {
    final bgRect = Rect.fromLTRB(0, 0, size.width, size.height);
    final bgRrect = RRect.fromRectXY(bgRect, radius, radius);
    if (bgColors.length == 1) {
      mBgPaint.shader = null;
      mBgPaint.color = bgColors[0];
    } else {
      mBgPaint.color = Colors.white;
      mBgPaint.shader = LinearGradient(
        colors: bgColors,
        transform: const GradientRotation(-pi / 2),
      ).createShader(bgRect);
    }
    if (colors.length == 1) {
      mPaint.shader = null;
      mPaint.color = colors[0];
    } else {
      mPaint.color = Colors.white;
      mPaint.shader = LinearGradient(
        colors: colors,
        transform: const GradientRotation(-pi / 2),
      ).createShader(bgRect);
    }

    mPaint.strokeWidth = strokeWidth;
    canvas.drawRRect(bgRrect, mBgPaint);

    double y = progress * size.height;
    if (y < radius) {
      double x = sqrt(radius * radius - (radius - y) * (radius - y));

      if (radius * 2 < size.width) {
        final rect1 = Rect.fromLTRB(0 + radius, (1 - progress) * size.height,
            size.width - radius, size.height);
        canvas.drawRect(rect1, mPaint);
      }

      Path pathL = Path();

      final rectL =
          Rect.fromLTRB(0, size.height - radius * 2, radius * 2, size.height);

      pathL.moveTo(radius, size.height);
      pathL.arcTo(rectL, 0.5 * pi, sin(y / radius) * 2, false);
      pathL.lineTo(radius - x, (1 - progress) * size.height);
      pathL.lineTo(radius, (1 - progress) * size.height);
      canvas.drawPath(pathL, mPaint);

      Path pathR = Path();

      final rectR = Rect.fromLTRB(size.width - radius * 2,
          size.height - radius * 2, size.width, size.height);

      pathR.moveTo(size.width - radius, size.height);
      pathR.arcTo(rectR, 0.5 * pi, -sin(y / radius) * 2, false);
      pathR.lineTo(size.width - radius + x, (1 - progress) * size.height);
      pathR.lineTo(size.width - radius, (1 - progress) * size.height);
      canvas.drawPath(pathR, mPaint);
    } else {
      final rect = Rect.fromLTRB(0, (1 - progress) * size.height, size.width,
          size.height - radius + 1);
      final rrect = RRect.fromRectAndCorners(rect,
          topLeft: Radius.circular(radius), topRight: Radius.circular(radius));
      canvas.drawRRect(rrect, mPaint);
      if (radius * 2 < size.width) {
        final rect1 = Rect.fromLTRB(
            0 + radius, size.height - radius, size.width - radius, size.height);
        canvas.drawRect(rect1, mPaint);
      }

      final rectL =
          Rect.fromLTRB(0, size.height - radius * 2, radius * 2, size.height);
      final rectR = Rect.fromLTRB(size.width - radius * 2,
          size.height - radius * 2, size.width, size.height);
      canvas.drawArc(rectL, 0.5 * pi, 0.5 * pi, true, mPaint);
      canvas.drawArc(rectR, 0.5 * pi, -0.5 * pi, true, mPaint);
    }
  }

  @override
  bool shouldRepaint(covariant _VerticalProgressPainter oldDelegate) {
    return true;
  }
}
